

-- left: move circle left
-- right: move circle right
-- space: reset circle to start position


local shapes = require 'collision_shapes'
local vec2 = require 'vec2'

local ground = {
	{
		x = 50,
		y = 50,
		shape = shapes.new_box_shape(0, 0, 75, 225),
	},
	{
		x = 125,
		y = 50,
		shape = shapes.new_polygon_shape(0,0, 175,150, 175,225, 0,225),
	},
	{
		x = 300,
		y = 200,
		shape = shapes.new_box_shape(0, 0, 75, 75),
	},
}

local obj = {
	x = 175,
	y = 25,
	vx = 0,
	vy = 0,
	shape = shapes.new_circle_shape(20),
}

function love.update(dt)
	obj.vx =
		(love.keyboard.isDown('left') and -150 or 0) +
		(love.keyboard.isDown('right') and 150 or 0)

	local old_vx, old_vy = obj.vx, obj.vy
	obj.vy = obj.vy + 1500 * dt
	obj.x, obj.y = vec2.add(obj.x, obj.y, vec2.scale(dt, vec2.interp(0.5, old_vx,old_vy, obj.vx,obj.vy)))

	for _, solid in ipairs(ground) do
		local hit, dx,dy = obj.shape:intersects(solid.shape, obj.x,obj.y,1,0, solid.x,solid.y,1,0)
		if hit then
			obj.x = obj.x + dx
			obj.y = obj.y + dy
			local dot = vec2.dot(0, -1, vec2.norm(dx, dy))
			if dot > 0.5 then
				obj.vy = math.min(0, obj.vy)
			end
		end
	end

	if love.keyboard.isDown(' ') then
		obj.x, obj.y = 175, 25
		obj.vx, obj.vy = 0, 0
	end
end

function love.draw()
	love.graphics.setColor(210, 160, 90)
	for _, solid in ipairs(ground) do
		love.graphics.push()
		love.graphics.translate(solid.x, solid.y)
		solid.shape:debug_draw('line')
		love.graphics.pop()
	end

	love.graphics.setColor(0, 255, 220)
	love.graphics.push()
	love.graphics.translate(obj.x, obj.y)
	obj.shape:debug_draw('line')
	love.graphics.pop()
end

