
local vec2 = {}

local sin, cos, atan2, sqrt = math.sin, math.cos, math.atan2, math.sqrt

function vec2.add(ax,ay, bx,by)
	return ax + bx, ay + by
end

function vec2.sub(ax,ay, bx,by)
	return ax - bx, ay - by
end

function vec2.diff(ax,ay, bx,by)
	return bx - ax, by - ay
end

function vec2.scale(s, x,y)
	return s * x, s * y
end

function vec2.inv_scale(s, x,y)
	return x / s, y / s
end

function vec2.dot(ax,ay, bx,by)
	return ax*bx + ay*by
end

function vec2.cross(ax,ay, bx,by)
	return ax*by - ay*bx
end

function vec2.interp(pos, ax,ay, bx,by)
	return ax + (bx - ax) * pos, ay + (by - ay) * pos
end

function vec2.to_x_axis(axis_x, axis_y, x, y)
	return vec2.dot(x, y, axis_x, axis_y), vec2.dot(x, y, vec2.perp(axis_x, axis_y))
end

function vec2.from_x_axis(axis_x, axis_y, x, y)
	return vec2.add(axis_x * x, axis_y * x, vec2.scale(y, vec2.perp(axis_x, axis_y)))
end

function vec2.to_y_axis(axis_x, axis_y, x, y)
	return vec2.dot(x, y, vec2.neg_perp(axis_x, axis_y)), vec2.dot(x, y, axis_x, axis_y)
end

function vec2.from_y_axis(axis_x, axis_y, x, y)
	return vec2.add(axis_x * y, axis_y * y, vec2.scale(x, vec2.neg_perp(axis_x, axis_y)))
end

function vec2.to_new_x_axis(sx,sy, ax,ay)
	return
		ax * sx + ay * sy,
		ax * sy - ay * sx
end

function vec2.from_new_x_axis(sx,sy, ax,ay)
	return
		ax * sx - ay * sy,
		ay * sx + ax * sy
end

function vec2.to_new_y_axis(sx,sy, ax,ay)
	return
		ay * sx - ax * sy,
		ax * sx + ay * sy
end

function vec2.from_new_y_axis(sx,sy, ax,ay)
	return
		ax * sy + ay * sx,
		ay * sy - ax * sx
end

function vec2.rotate(angle, x,y)
	local s, c = sin(angle), cos(angle)
	return x*c-y*s, x*s+y*c
end

function vec2.rotate_around(angle, ox,oy, x,y)
	local s, c = sin(angle), cos(angle)
	x,y = x-ox, y-oy
	return ox+x*c-y*s, oy+x*s+y*c
end

function vec2.perp(x,y)
	return -y, x
end

function vec2.neg_perp(x,y)
	return y, -x
end

function vec2.len(x,y)
	return sqrt(x*x + y*y)
end
vec2.length = vec2.len

function vec2.len2(x,y)
	return x*x + y*y
end
vec2.length2 = vec2.len2

function vec2.norm(x,y)
	local length = sqrt(x*x + y*y)
	return x/length, y/length
end
vec2.normalize = vec2.norm

function vec2.safe_norm(x,y)
	if x==0 and y==0 then return 1, 0 end
	local length = sqrt(x*x + y*y)
	return x/length, y/length
end
vec2.safe_normalize = vec2.safe_norm

function vec2.abs_scale(s, x,y)
	local length = sqrt(x*x + y*y)/s
	return x/length, y/length
end

function vec2.safe_abs_scale(s, x,y)
	if x==0 and y==0 then return s, 0 end
	local length = sqrt(x*x + y*y)/s
	return x/length, y/length
end

function vec2.from_polar(angle, length)
	length = length or 1
	return cos(angle)*length, sin(angle)*length
end

function vec2.angle(x, y)
	return atan2(y, x)
end

function vec2.project_onto(ax,ay, tx,ty)
	local length = ax*tx + ay*ty
	local px, py = tx*length, ty*length
	return px, py, ax-px, ay-py
end

function vec2.clamp_len(maxlen, x,y)
	if x*x+y*y > maxlen*maxlen then
		return vec2.abs_scale(maxlen, x,y)
	end
	return x,y
end

return vec2
